@@ -1,9 +1,11 @@
 import json
 import os
+import time
 from glob import glob
 
 import yaml
 from twacapic.auth import get_api
+from TwitterAPI import TwitterConnectionError, TwitterRequestError
 
 
 class UserGroup:
@@ -26,7 +28,9 @@ class UserGroup:
 
     def request_tweets(self, api, user_id, params, get_all_pages=False):
 
+        @retry
         def get_page(params):
+
             response = api.request(f'users/:{user_id}/tweets', params)
 
             assert response.status_code == 200
@@ -70,11 +74,14 @@ class UserGroup:
             if not os.path.isfile(meta_file_path):
 
                 params = {'max_results': max_results_per_call}
-                oldest_id, newest_id = self.request_tweets(api, user_id, params)
+                collected_ids = self.request_tweets(api, user_id, params)
 
                 user_metadata = {}
-                user_metadata['newest_id'] = newest_id
-                user_metadata['oldest_id'] = oldest_id
+
+                if collected_ids is not None:
+                    oldest_id, newest_id = collected_ids
+                    user_metadata['newest_id'] = newest_id
+                    user_metadata['oldest_id'] = oldest_id
 
                 with open(meta_file_path, 'w') as metafile:
                     yaml.dump(user_metadata, metafile)
@@ -111,3 +118,39 @@ class UserGroup:
             with open(f'{self.path}/{user_id}/meta.yaml', 'r') as f:
                 meta[user_id] = yaml.safe_load(f)
         return meta
+
+
+def retry(func):
+
+    def retried_func(*args, **kwargs):
+        max_tries = 10
+        tries = 0
+        total_sleep_seconds = 0
+
+        while True:
+            try:
+                resp = func(*args, **kwargs)
+
+            except (TwitterConnectionError, TwitterRequestError, AssertionError) as e:
+
+                print(e)
+
+                if tries < max_tries:
+
+                    tries += 1
+
+                    sleep_seconds = min((tries ** 2), max(900 - total_sleep_seconds, 30))
+                    total_sleep_seconds = total_sleep_seconds + sleep_seconds
+                else:
+                    print('Maximum retries reached. Raising Exception …')
+                    raise e
+
+                print(f"Retry in {sleep_seconds} seconds …")
+                time.sleep(sleep_seconds)
+                continue
+
+            break
+
+        return resp
+
+    return retried_func
